# r e g i o n   P D F s h a r p   -   A   . N E T   l i b r a r y   f o r   p r o c e s s i n g   P D F 
 
 / / 
 
 / /   A u t h o r s : 
 
 / /       S t e f a n   L a n g e   ( m a i l t o : S t e f a n . L a n g e @ p d f s h a r p . c o m ) 
 
 / / 
 
 / /   C o p y r i g h t   ( c )   2 0 0 5 - 2 0 0 9   e m p i r a   S o f t w a r e   G m b H ,   C o l o g n e   ( G e r m a n y ) 
 
 / / 
 
 / /   h t t p : / / w w w . p d f s h a r p . c o m 
 
 / /   h t t p : / / s o u r c e f o r g e . n e t / p r o j e c t s / p d f s h a r p 
 
 / / 
 
 / /   P e r m i s s i o n   i s   h e r e b y   g r a n t e d ,   f r e e   o f   c h a r g e ,   t o   a n y   p e r s o n   o b t a i n i n g   a 
 
 / /   c o p y   o f   t h i s   s o f t w a r e   a n d   a s s o c i a t e d   d o c u m e n t a t i o n   f i l e s   ( t h e   " S o f t w a r e " ) , 
 
 / /   t o   d e a l   i n   t h e   S o f t w a r e   w i t h o u t   r e s t r i c t i o n ,   i n c l u d i n g   w i t h o u t   l i m i t a t i o n 
 
 / /   t h e   r i g h t s   t o   u s e ,   c o p y ,   m o d i f y ,   m e r g e ,   p u b l i s h ,   d i s t r i b u t e ,   s u b l i c e n s e , 
 
 / /   a n d / o r   s e l l   c o p i e s   o f   t h e   S o f t w a r e ,   a n d   t o   p e r m i t   p e r s o n s   t o   w h o m   t h e 
 
 / /   S o f t w a r e   i s   f u r n i s h e d   t o   d o   s o ,   s u b j e c t   t o   t h e   f o l l o w i n g   c o n d i t i o n s : 
 
 / / 
 
 / /   T h e   a b o v e   c o p y r i g h t   n o t i c e   a n d   t h i s   p e r m i s s i o n   n o t i c e   s h a l l   b e   i n c l u d e d 
 
 / /   i n   a l l   c o p i e s   o r   s u b s t a n t i a l   p o r t i o n s   o f   t h e   S o f t w a r e . 
 
 / / 
 
 / /   T H E   S O F T W A R E   I S   P R O V I D E D   " A S   I S " ,   W I T H O U T   W A R R A N T Y   O F   A N Y   K I N D ,   E X P R E S S   O R 
 
 / /   I M P L I E D ,   I N C L U D I N G   B U T   N O T   L I M I T E D   T O   T H E   W A R R A N T I E S   O F   M E R C H A N T A B I L I T Y , 
 
 / /   F I T N E S S   F O R   A   P A R T I C U L A R   P U R P O S E   A N D   N O N I N F R I N G E M E N T .   I N   N O   E V E N T   S H A L L 
 
 / /   T H E   A U T H O R S   O R   C O P Y R I G H T   H O L D E R S   B E   L I A B L E   F O R   A N Y   C L A I M ,   D A M A G E S   O R   O T H E R 
 
 / /   L I A B I L I T Y ,   W H E T H E R   I N   A N   A C T I O N   O F   C O N T R A C T ,   T O R T   O R   O T H E R W I S E ,   A R I S I N G 
 
 / /   F R O M ,   O U T   O F   O R   I N   C O N N E C T I O N   W I T H   T H E   S O F T W A R E   O R   T H E   U S E   O R   O T H E R   
 
 / /   D E A L I N G S   I N   T H E   S O F T W A R E . 
 
 # e n d r e g i o n 
 
 
 
 u s i n g   S y s t e m ; 
 
 u s i n g   S y s t e m . G l o b a l i z a t i o n ; 
 
 u s i n g   S y s t e m . D i a g n o s t i c s ; 
 
 u s i n g   S y s t e m . C o l l e c t i o n s ; 
 
 u s i n g   S y s t e m . T e x t ; 
 
 u s i n g   S y s t e m . I O ; 
 
 u s i n g   P d f S h a r p . P d f . I n t e r n a l ; 
 
 
 
 n a m e s p a c e   P d f S h a r p . P d f . C o n t e n t 
 
 { 
 
     / / /   < s u m m a r y > 
 
     / / /   L e x i c a l   a n a l y z e r   f o r   P D F   c o n t e n t   f i l e s .   A d o b e   s p e c i f i e s   n o   g r a m m a r ,   b u t   i t   s e e m s   t h a t   i t 
 
     / / /   i s   a   s i m p l e   p o s t - f i x   n o t a t i o n . 
 
     / / /   < / s u m m a r y > 
 
     i n t e r n a l   c l a s s   C L e x e r 
 
     { 
 
         / / /   < s u m m a r y > 
 
         / / /   I n i t i a l i z e s   a   n e w   i n s t a n c e   o f   t h e   L e x e r   c l a s s . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C L e x e r ( b y t e [ ]   c o n t e n t ) 
 
         { 
 
             t h i s . c o n t e n t   =   c o n t e n t ; 
 
             t h i s . c h a r I n d e x   =   0 ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   R e a d s   t h e   n e x t   t o k e n   a n d   r e t u r n s   i t s   t y p e . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S c a n N e x t T o k e n ( ) 
 
         { 
 
         A g a i n : 
 
             C l e a r T o k e n ( ) ; 
 
             c h a r   c h   =   M o v e T o N o n W h i t e S p a c e ( ) ; 
 
             s w i t c h   ( c h ) 
 
             { 
 
                 c a s e   ' % ' : 
 
                     / /   E a t   c o m m e n t s ,   t h e   p a r s e r   d o e s n ' t   h a n d l e   t h e m 
 
                     / / r e t u r n   t h i s . s y m b o l   =   S c a n C o m m e n t ( ) ; 
 
                     S c a n C o m m e n t ( ) ; 
 
                     g o t o   A g a i n ; 
 
 
 
                 c a s e   ' / ' : 
 
                     r e t u r n   t h i s . s y m b o l   =   S c a n N a m e ( ) ; 
 
 
 
                 / / c a s e   ' R ' : 
 
                 / /     i f   ( L e x e r . I s W h i t e S p a c e ( t h i s . n e x t C h a r ) ) 
 
                 / /     { 
 
                 / /         S c a n N e x t C h a r ( ) ; 
 
                 / /         r e t u r n   S y m b o l . R ; 
 
                 / /     } 
 
                 / /     b r e a k ; 
 
 
 
                 c a s e   ' + ' : 
 
                 c a s e   ' - ' : 
 
                     r e t u r n   t h i s . s y m b o l   =   S c a n N u m b e r ( ) ; 
 
 
 
                 c a s e   ' [ ' : 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     r e t u r n   t h i s . s y m b o l   =   C S y m b o l . B e g i n A r r a y ; 
 
 
 
                 c a s e   ' ] ' : 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     r e t u r n   t h i s . s y m b o l   =   C S y m b o l . E n d A r r a y ; 
 
 
 
                 c a s e   ' ( ' : 
 
                     r e t u r n   t h i s . s y m b o l   =   S c a n L i t e r a l S t r i n g ( ) ; 
 
 
 
                 c a s e   ' < ' : 
 
                     r e t u r n   t h i s . s y m b o l   =   S c a n H e x a d e c i m a l S t r i n g ( ) ; 
 
 
 
                 c a s e   ' . ' : 
 
                     r e t u r n   t h i s . s y m b o l   =   S c a n N u m b e r ( ) ; 
 
 
 
                 c a s e   ' " ' : 
 
                 c a s e   ' \ ' ' : 
 
                     r e t u r n   t h i s . s y m b o l   =   S c a n O p e r a t o r ( ) ; 
 
             } 
 
             i f   ( C h a r . I s D i g i t ( c h ) ) 
 
                 r e t u r n   t h i s . s y m b o l   =   S c a n N u m b e r ( ) ; 
 
 
 
             i f   ( C h a r . I s L e t t e r ( c h ) ) 
 
                 r e t u r n   t h i s . s y m b o l   =   S c a n O p e r a t o r ( ) ; 
 
 
 
             i f   ( c h   = =   C h a r s . E O F ) 
 
                 r e t u r n   t h i s . s y m b o l   =   C S y m b o l . E o f ; 
 
 
 
             D e b u g . A s s e r t ( f a l s e ,   " U n e x p e c t e d   c h a r a c t e r   i n   c o n t e n t   s t r e a m   ( m a y b e   n o t   i m p l e m e n t e d ) . " ) ; 
 
             r e t u r n   t h i s . s y m b o l   =   C S y m b o l . N o n e ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   S c a n s   a   c o m m e n t   l i n e .   ( N o t   y e t   u s e d ,   c o m m e n t s   a r e   s k i p p e d   b y   l e x e r . ) 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S c a n C o m m e n t ( ) 
 
         { 
 
             D e b u g . A s s e r t ( t h i s . c u r r C h a r   = =   C h a r s . P e r c e n t ) ; 
 
 
 
             C l e a r T o k e n ( ) ; 
 
             c h a r   c h ; 
 
             w h i l e   ( ( c h   =   A p p e n d A n d S c a n N e x t C h a r ( ) )   ! =   C h a r s . L F   & &   c h   ! =   C h a r s . E O F )   {   } 
 
             r e t u r n   t h i s . s y m b o l   =   C S y m b o l . C o m m e n t ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   S c a n s   t h e   b y t e s   o f   a n   i n l i n e   i m a g e 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S c a n I n l i n e I m a g e ( ) 
 
         { 
 
             / /   T O D O :   
 
             w h i l e   ( t h i s . c u r r C h a r   ! =   ' E '   & &   t h i s . n e x t C h a r   ! =   ' I ' ) 
 
                 S c a n N e x t C h a r ( ) ; 
 
             r e t u r n   C S y m b o l . N o n e ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   S c a n s   a   n a m e . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S c a n N a m e ( ) 
 
         { 
 
             D e b u g . A s s e r t ( t h i s . c u r r C h a r   = =   C h a r s . S l a s h ) ; 
 
 
 
             C l e a r T o k e n ( ) ; 
 
             w h i l e   ( t r u e ) 
 
             { 
 
                 c h a r   c h   =   A p p e n d A n d S c a n N e x t C h a r ( ) ; 
 
                 i f   ( I s W h i t e S p a c e ( c h )   | |   I s D e l i m i t e r ( c h ) ) 
 
                     r e t u r n   t h i s . s y m b o l   =   C S y m b o l . N a m e ; 
 
                 
 
                 i f   ( c h   = =   ' # ' ) 
 
                 { 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     c h a r [ ]   h e x   =   n e w   c h a r [ 2 ] ; 
 
                     h e x [ 0 ]   =   t h i s . c u r r C h a r ; 
 
                     h e x [ 1 ]   =   t h i s . n e x t C h a r ; 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     / /   T O D O   C h e c k   s y n t a x 
 
                     c h   =   ( c h a r ) ( u s h o r t ) i n t . P a r s e ( n e w   s t r i n g ( h e x ) ,   N u m b e r S t y l e s . A l l o w H e x S p e c i f i e r ) ; 
 
                     t h i s . c u r r C h a r   =   c h ; 
 
                 } 
 
             } 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   S c a n s   a n   i n t e g e r   o r   r e a l   n u m b e r . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S c a n N u m b e r ( ) 
 
         { 
 
             l o n g   v a l u e   =   0 ; 
 
             i n t   d e c i m a l D i g i t s   =   0 ; 
 
             b o o l   p e r i o d   =   f a l s e ; 
 
             b o o l   n e g a t i v e   =   f a l s e ; 
 
 
 
             C l e a r T o k e n ( ) ; 
 
             c h a r   c h   =   t h i s . c u r r C h a r ; 
 
             i f   ( c h   = =   ' + '   | |   c h   = =   ' - ' ) 
 
             { 
 
                 i f   ( c h   = =   ' - ' ) 
 
                     n e g a t i v e   =   t r u e ; 
 
                 t h i s . t o k e n . A p p e n d ( c h ) ; 
 
                 c h   =   S c a n N e x t C h a r ( ) ; 
 
             } 
 
             w h i l e   ( t r u e ) 
 
             { 
 
                 i f   ( c h a r . I s D i g i t ( c h ) ) 
 
                 { 
 
                     t h i s . t o k e n . A p p e n d ( c h ) ; 
 
                     i f   ( d e c i m a l D i g i t s   <   1 0 ) 
 
                     { 
 
                         v a l u e   =   1 0   *   v a l u e   +   c h   -   ' 0 ' ; 
 
                         i f   ( p e r i o d ) 
 
                             d e c i m a l D i g i t s + + ; 
 
                     } 
 
                 } 
 
                 e l s e   i f   ( c h   = =   ' . ' ) 
 
                 { 
 
                     i f   ( p e r i o d ) 
 
                         t h r o w   n e w   C o n t e n t R e a d e r E x c e p t i o n ( " M o r e   t h a n   o n e   p e r i o d   i n   n u m b e r . " ) ; 
 
                     p e r i o d   =   t r u e ; 
 
                     t h i s . t o k e n . A p p e n d ( c h ) ; 
 
                 } 
 
                 e l s e 
 
                     b r e a k ; 
 
                 c h   =   S c a n N e x t C h a r ( ) ; 
 
             } 
 
 
 
             i f   ( n e g a t i v e ) 
 
                 v a l u e   =   - v a l u e ; 
 
             i f   ( p e r i o d ) 
 
             { 
 
                 i f   ( d e c i m a l D i g i t s   >   0 ) 
 
                 { 
 
                     t o k e n A s R e a l   =   v a l u e   /   P o w e r s O f 1 0 [ d e c i m a l D i g i t s ] ; 
 
                     / / t h i s . t o k e n A s L o n g   =   v a l u e   /   P o w e r s O f 1 0 [ d e c i m a l D i g i t s ] ; 
 
                 } 
 
                 e l s e 
 
                 { 
 
                     t h i s . t o k e n A s R e a l   =   v a l u e ; 
 
                     t h i s . t o k e n A s L o n g   =   v a l u e ; 
 
                 } 
 
                 r e t u r n   C S y m b o l . R e a l ; 
 
             } 
 
             t h i s . t o k e n A s L o n g   =   v a l u e ; 
 
             t h i s . t o k e n A s R e a l   =   C o n v e r t . T o D o u b l e ( v a l u e ) ; 
 
 
 
             / / l o n g   l   =   I n t 6 4 . P a r s e ( t h i s . t o k e n . T o S t r i n g ( ) ,   C u l t u r e I n f o . I n v a r i a n t C u l t u r e ) ; 
 
             D e b u g . A s s e r t ( I n t 6 4 . P a r s e ( t h i s . t o k e n . T o S t r i n g ( ) ,   C u l t u r e I n f o . I n v a r i a n t C u l t u r e )   = =   v a l u e ) ; 
 
 
 
             i f   ( v a l u e   > =   I n t 3 2 . M i n V a l u e   & &   v a l u e   < =   I n t 3 2 . M a x V a l u e ) 
 
                 r e t u r n   C S y m b o l . I n t e g e r ; 
 
             / / i f   ( l   >   0   & &   l   < =   U I n t 3 2 . M a x V a l u e ) 
 
             / /     r e t u r n   S y m b o l . U I n t e g e r ; 
 
             t h r o w   n e w   C o n t e n t R e a d e r E x c e p t i o n ( " N u m b e r   e x c e e d s   i n t e g e r   r a n g e . " ) ; 
 
         } 
 
         s t a t i c   r e a d o n l y   d o u b l e [ ]   P o w e r s O f 1 0   =   n e w   d o u b l e [ ]   {   1 ,   1 0 ,   1 0 0 ,   1 0 0 0 ,   1 0 0 0 0 ,   1 0 0 0 0 0 ,   1 0 0 0 0 0 0 ,   1 0 0 0 0 0 0 0 ,   1 0 0 0 0 0 0 0 0 ,   1 0 0 0 0 0 0 0 0 0   } ; 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   S c a n s   a n   o p e r a t o r . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S c a n O p e r a t o r ( ) 
 
         { 
 
             C l e a r T o k e n ( ) ; 
 
             c h a r   c h   =   t h i s . c u r r C h a r ; 
 
             / /   S c a n   t o k e n 
 
             w h i l e   ( I s O p e r a t o r C h a r ( c h ) ) 
 
                 c h   =   A p p e n d A n d S c a n N e x t C h a r ( ) ; 
 
 
 
             r e t u r n   t h i s . s y m b o l   =   C S y m b o l . O p e r a t o r ; 
 
         } 
 
 
 
         / /   T O D O 
 
         p u b l i c   C S y m b o l   S c a n L i t e r a l S t r i n g ( ) 
 
         { 
 
             D e b u g . A s s e r t ( t h i s . c u r r C h a r   = =   C h a r s . P a r e n L e f t ) ; 
 
 
 
             C l e a r T o k e n ( ) ; 
 
             i n t   p a r e n L e v e l   =   0 ; 
 
             c h a r   c h   =   S c a n N e x t C h a r ( ) ; 
 
             / /   T e s t   U N I C O D E   s t r i n g 
 
             i f   ( c h   = =   ' \ x F E '   & &   t h i s . n e x t C h a r   = =   ' \ x F F ' ) 
 
             { 
 
                 / /   I ' m   n o t   s u r e   i f   t h e   c o d e   i s   c o r r e c t   i n   a n y   c a s e . 
 
                 / /   ?   C a n   a   U N I C O D E   c h a r a c t e r   n o t   s t a r t   w i t h   ' ) '   a s   h i b y t e 
 
                 / /   ?   W h a t   a b o u t   \ #   e s c a p e   s e q u e n c e s 
 
                 S c a n N e x t C h a r ( ) ; 
 
                 c h a r   c h H i   =   S c a n N e x t C h a r ( ) ; 
 
                 i f   ( c h H i   = =   ' ) ' ) 
 
                 { 
 
                     / /   T h e   e m p t y   u n i c o d e   s t r i n g . . . 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     r e t u r n   t h i s . s y m b o l   =   C S y m b o l . S t r i n g ; 
 
                 } 
 
                 c h a r   c h L o   =   S c a n N e x t C h a r ( ) ; 
 
                 c h   =   ( c h a r ) ( c h H i   *   2 5 6   +   c h L o ) ; 
 
                 w h i l e   ( t r u e ) 
 
                 { 
 
                 S k i p C h a r : 
 
                     s w i t c h   ( c h ) 
 
                     { 
 
                         c a s e   ' ( ' : 
 
                             p a r e n L e v e l + + ; 
 
                             b r e a k ; 
 
 
 
                         c a s e   ' ) ' : 
 
                             i f   ( p a r e n L e v e l   = =   0 ) 
 
                             { 
 
                                 S c a n N e x t C h a r ( ) ; 
 
                                 r e t u r n   t h i s . s y m b o l   =   C S y m b o l . S t r i n g ; 
 
                             } 
 
                             p a r e n L e v e l - - ; 
 
                             b r e a k ; 
 
 
 
                         c a s e   ' \ \ ' : 
 
                             { 
 
                                 / /   T O D O :   n o t   s u r e   t h a t   t h i s   i s   c o r r e c t . . . 
 
                                 c h   =   S c a n N e x t C h a r ( ) ; 
 
                                 s w i t c h   ( c h ) 
 
                                 { 
 
                                     c a s e   ' n ' : 
 
                                         c h   =   C h a r s . L F ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' r ' : 
 
                                         c h   =   C h a r s . C R ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' t ' : 
 
                                         c h   =   C h a r s . H T ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' b ' : 
 
                                         c h   =   C h a r s . B S ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' f ' : 
 
                                         c h   =   C h a r s . F F ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' ( ' : 
 
                                         c h   =   C h a r s . P a r e n L e f t ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' ) ' : 
 
                                         c h   =   C h a r s . P a r e n R i g h t ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' \ \ ' : 
 
                                         c h   =   C h a r s . B a c k S l a s h ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   C h a r s . L F : 
 
                                         c h   =   S c a n N e x t C h a r ( ) ; 
 
                                         g o t o   S k i p C h a r ; 
 
 
 
                                     d e f a u l t : 
 
                                         i f   ( C h a r . I s D i g i t ( c h ) ) 
 
                                         { 
 
                                             / /   O c t a l   c h a r a c t e r   c o d e 
 
                                             i n t   n   =   c h   -   ' 0 ' ; 
 
                                             i f   ( C h a r . I s D i g i t ( t h i s . n e x t C h a r ) ) 
 
                                             { 
 
                                                 n   =   n   *   8   +   S c a n N e x t C h a r ( )   -   ' 0 ' ; 
 
                                                 i f   ( C h a r . I s D i g i t ( t h i s . n e x t C h a r ) ) 
 
                                                     n   =   n   *   8   +   S c a n N e x t C h a r ( )   -   ' 0 ' ; 
 
                                             } 
 
                                             c h   =   ( c h a r ) n ; 
 
                                         } 
 
                                         b r e a k ; 
 
                                 } 
 
                                 b r e a k ; 
 
                             } 
 
 
 
                         / /   T O D O   ? ? ? 
 
                         / / c a s e   ' # ' : 
 
                         / /     D e b u g . A s s e r t ( f a l s e ,   " N o t   y e t   i m p l e m e n t e d " ) ; 
 
                         / /     b r e a k ; 
 
 
 
                         d e f a u l t : 
 
                             b r e a k ; 
 
                     } 
 
                     t h i s . t o k e n . A p p e n d ( c h ) ; 
 
                     c h H i   =   S c a n N e x t C h a r ( ) ; 
 
                     i f   ( c h H i   = =   ' ) ' ) 
 
                     { 
 
                         S c a n N e x t C h a r ( ) ; 
 
                         r e t u r n   t h i s . s y m b o l   =   C S y m b o l . S t r i n g ; 
 
                     } 
 
                     c h L o   =   S c a n N e x t C h a r ( ) ; 
 
                     c h   =   ( c h a r ) ( c h H i   *   2 5 6   +   c h L o ) ; 
 
                 } 
 
             } 
 
             e l s e 
 
             { 
 
                 / /   8 - b i t   c h a r a c t e r s 
 
                 w h i l e   ( t r u e ) 
 
                 { 
 
                 S k i p C h a r : 
 
                     s w i t c h   ( c h ) 
 
                     { 
 
                         c a s e   ' ( ' : 
 
                             p a r e n L e v e l + + ; 
 
                             b r e a k ; 
 
 
 
                         c a s e   ' ) ' : 
 
                             i f   ( p a r e n L e v e l   = =   0 ) 
 
                             { 
 
                                 S c a n N e x t C h a r ( ) ; 
 
                                 r e t u r n   t h i s . s y m b o l   =   C S y m b o l . S t r i n g ; 
 
                             } 
 
                             p a r e n L e v e l - - ; 
 
                             b r e a k ; 
 
 
 
                         c a s e   ' \ \ ' : 
 
                             { 
 
                                 c h   =   S c a n N e x t C h a r ( ) ; 
 
                                 s w i t c h   ( c h ) 
 
                                 { 
 
                                     c a s e   ' n ' : 
 
                                         c h   =   C h a r s . L F ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' r ' : 
 
                                         c h   =   C h a r s . C R ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' t ' : 
 
                                         c h   =   C h a r s . H T ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' b ' : 
 
                                         c h   =   C h a r s . B S ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' f ' : 
 
                                         c h   =   C h a r s . F F ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' ( ' : 
 
                                         c h   =   C h a r s . P a r e n L e f t ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' ) ' : 
 
                                         c h   =   C h a r s . P a r e n R i g h t ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   ' \ \ ' : 
 
                                         c h   =   C h a r s . B a c k S l a s h ; 
 
                                         b r e a k ; 
 
 
 
                                     c a s e   C h a r s . L F : 
 
                                         c h   =   S c a n N e x t C h a r ( ) ; 
 
                                         g o t o   S k i p C h a r ; 
 
 
 
                                     d e f a u l t : 
 
                                         i f   ( C h a r . I s D i g i t ( c h ) ) 
 
                                         { 
 
                                             / /   O c t a l   c h a r a c t e r   c o d e 
 
                                             i n t   n   =   c h   -   ' 0 ' ; 
 
                                             i f   ( C h a r . I s D i g i t ( t h i s . n e x t C h a r ) ) 
 
                                             { 
 
                                                 n   =   n   *   8   +   S c a n N e x t C h a r ( )   -   ' 0 ' ; 
 
                                                 i f   ( C h a r . I s D i g i t ( t h i s . n e x t C h a r ) ) 
 
                                                     n   =   n   *   8   +   S c a n N e x t C h a r ( )   -   ' 0 ' ; 
 
                                             } 
 
                                             c h   =   ( c h a r ) n ; 
 
                                         } 
 
                                         b r e a k ; 
 
                                 } 
 
                                 b r e a k ; 
 
                             } 
 
 
 
                         / /   T O D O   ? ? ? 
 
                         / / c a s e   ' # ' : 
 
                         / /     D e b u g . A s s e r t ( f a l s e ,   " N o t   y e t   i m p l e m e n t e d " ) ; 
 
                         / /     b r e a k ; 
 
 
 
                         d e f a u l t : 
 
                             b r e a k ; 
 
                     } 
 
                     t h i s . t o k e n . A p p e n d ( c h ) ; 
 
                     c h   =   S c a n N e x t C h a r ( ) ; 
 
                 } 
 
             } 
 
         } 
 
 
 
         / /   T O D O 
 
         p u b l i c   C S y m b o l   S c a n H e x a d e c i m a l S t r i n g ( ) 
 
         { 
 
             D e b u g . A s s e r t ( t h i s . c u r r C h a r   = =   C h a r s . L e s s ) ; 
 
 
 
             C l e a r T o k e n ( ) ; 
 
             c h a r [ ]   h e x   =   n e w   c h a r [ 2 ] ; 
 
             S c a n N e x t C h a r ( ) ; 
 
             w h i l e   ( t r u e ) 
 
             { 
 
                 M o v e T o N o n W h i t e S p a c e ( ) ; 
 
                 i f   ( t h i s . c u r r C h a r   = =   ' > ' ) 
 
                 { 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     b r e a k ; 
 
                 } 
 
                 i f   ( c h a r . I s L e t t e r O r D i g i t ( t h i s . c u r r C h a r ) ) 
 
                 { 
 
                     h e x [ 0 ]   =   c h a r . T o U p p e r ( t h i s . c u r r C h a r ) ; 
 
                     h e x [ 1 ]   =   c h a r . T o U p p e r ( t h i s . n e x t C h a r ) ; 
 
                     i n t   c h   =   i n t . P a r s e ( n e w   s t r i n g ( h e x ) ,   N u m b e r S t y l e s . A l l o w H e x S p e c i f i e r ) ; 
 
                     t h i s . t o k e n . A p p e n d ( C o n v e r t . T o C h a r ( c h ) ) ; 
 
                     S c a n N e x t C h a r ( ) ; 
 
                     S c a n N e x t C h a r ( ) ; 
 
                 } 
 
             } 
 
             s t r i n g   c h a r s   =   t h i s . t o k e n . T o S t r i n g ( ) ; 
 
             i n t   c o u n t   =   c h a r s . L e n g t h ; 
 
             i f   ( c o u n t   >   2   & &   c h a r s [ 0 ]   = =   ( c h a r ) 0 x F E   & &   c h a r s [ 1 ]   = =   ( c h a r ) 0 x F F ) 
 
             { 
 
                 D e b u g . A s s e r t ( c o u n t   %   2   = =   0 ) ; 
 
                 t h i s . t o k e n . L e n g t h   =   0 ; 
 
                 f o r   ( i n t   i d x   =   2 ;   i d x   <   c o u n t ;   i d x   + =   2 ) 
 
                     t h i s . t o k e n . A p p e n d ( ( c h a r ) ( c h a r s [ i d x ]   *   2 5 6   +   c h a r s [ i d x   +   1 ] ) ) ; 
 
             } 
 
             r e t u r n   t h i s . s y m b o l   =   C S y m b o l . H e x S t r i n g ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   M o v e   c u r r e n t   p o s i t i o n   o n e   c h a r a c t e r   f u r t h e r   i n   c o n t e n t   s t r e a m . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   c h a r   S c a n N e x t C h a r ( ) 
 
         { 
 
             i f   ( C o n t L e n g t h   < =   t h i s . c h a r I n d e x ) 
 
             { 
 
                 t h i s . c u r r C h a r   =   C h a r s . E O F ; 
 
                 t h i s . n e x t C h a r   =   C h a r s . E O F ; 
 
             } 
 
             e l s e 
 
             { 
 
                 t h i s . c u r r C h a r   =   t h i s . n e x t C h a r ; 
 
                 t h i s . n e x t C h a r   =   ( c h a r ) t h i s . c o n t e n t [ t h i s . c h a r I n d e x + + ] ; 
 
                 i f   ( t h i s . c u r r C h a r   = =   C h a r s . C R ) 
 
                 { 
 
                     i f   ( t h i s . n e x t C h a r   = =   C h a r s . L F ) 
 
                     { 
 
                         / /   T r e a t   C R   L F   a s   L F 
 
                         t h i s . c u r r C h a r   =   t h i s . n e x t C h a r ; 
 
                         i f   ( C o n t L e n g t h   < =   t h i s . c h a r I n d e x ) 
 
                             t h i s . n e x t C h a r   =   C h a r s . E O F ; 
 
                         e l s e 
 
                             t h i s . n e x t C h a r   =   ( c h a r ) t h i s . c o n t e n t [ t h i s . c h a r I n d e x + + ] ; 
 
                     } 
 
                     e l s e 
 
                     { 
 
                         / /   T r e a t   s i n g l e   C R   a s   L F 
 
                         t h i s . c u r r C h a r   =   C h a r s . L F ; 
 
                     } 
 
                 } 
 
             } 
 
             r e t u r n   c u r r C h a r ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   R e s e t s   t h e   c u r r e n t   t o k e n   t o   t h e   e m p t y   s t r i n g . 
 
         / / /   < / s u m m a r y > 
 
         v o i d   C l e a r T o k e n ( ) 
 
         { 
 
             t h i s . t o k e n . L e n g t h   =   0 ; 
 
             t h i s . t o k e n A s L o n g   =   0 ; 
 
             t h i s . t o k e n A s R e a l   =   0 ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   A p p e n d s   c u r r e n t   c h a r a c t e r   t o   t h e   t o k e n   a n d   r e a d s   n e x t   o n e . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   c h a r   A p p e n d A n d S c a n N e x t C h a r ( ) 
 
         { 
 
             t o k e n . A p p e n d ( t h i s . c u r r C h a r ) ; 
 
             r e t u r n   S c a n N e x t C h a r ( ) ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   I f   t h e   c u r r e n t   c h a r a c t e r   i s   n o t   a   w h i t e   s p a c e ,   t h e   f u n c t i o n   i m m e d i a t e l y   r e t u r n s   i t . 
 
         / / /   O t h e r w i s e   t h e   P D F   c u r s o r   i s   m o v e d   f o r w a r d   t o   t h e   f i r s t   n o n - w h i t e   s p a c e   o r   E O F . 
 
         / / /   W h i t e   s p a c e s   a r e   N U L ,   H T ,   L F ,   F F ,   C R ,   a n d   S P . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   c h a r   M o v e T o N o n W h i t e S p a c e ( ) 
 
         { 
 
             w h i l e   ( t h i s . c u r r C h a r   ! =   C h a r s . E O F ) 
 
             { 
 
                 s w i t c h   ( t h i s . c u r r C h a r ) 
 
                 { 
 
                     c a s e   C h a r s . N U L : 
 
                     c a s e   C h a r s . H T : 
 
                     c a s e   C h a r s . L F : 
 
                     c a s e   C h a r s . F F : 
 
                     c a s e   C h a r s . C R : 
 
                     c a s e   C h a r s . S P : 
 
                         S c a n N e x t C h a r ( ) ; 
 
                         b r e a k ; 
 
 
 
                     d e f a u l t : 
 
                         r e t u r n   c u r r C h a r ; 
 
                 } 
 
             } 
 
             r e t u r n   c u r r C h a r ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   G e t s   o r   s e t s   t h e   c u r r e n t   s y m b o l . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   C S y m b o l   S y m b o l 
 
         { 
 
             g e t   {   r e t u r n   t h i s . s y m b o l ;   } 
 
             s e t   {   t h i s . s y m b o l   =   v a l u e ;   } 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   G e t s   t h e   c u r r e n t   t o k e n . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   s t r i n g   T o k e n 
 
         { 
 
             g e t   {   r e t u r n   t h i s . t o k e n . T o S t r i n g ( ) ;   } 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   I n t e r p r e t s   c u r r e n t   t o k e n   a s   i n t e g e r   l i t e r a l . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   i n t   T o k e n T o I n t e g e r 
 
         { 
 
             g e t 
 
             { 
 
                 D e b u g . A s s e r t ( t o k e n A s L o n g   = =   I n t 3 2 . P a r s e ( t h i s . t o k e n . T o S t r i n g ( ) ,   C u l t u r e I n f o . I n v a r i a n t C u l t u r e ) ) ; 
 
                 / / r e t u r n   I n t 3 2 . P a r s e ( t h i s . t o k e n . T o S t r i n g ( ) ,   C u l t u r e I n f o . I n v a r i a n t C u l t u r e ) ; 
 
                 r e t u r n   ( i n t ) t h i s . t o k e n A s L o n g ; 
 
             } 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   I n t e r p r e t   c u r r e n t   t o k e n   a s   r e a l   o r   i n t e g e r   l i t e r a l . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   d o u b l e   T o k e n T o R e a l 
 
         { 
 
             g e t 
 
             { 
 
                 D e b u g . A s s e r t ( t o k e n A s R e a l   = =   D o u b l e . P a r s e ( t o k e n . T o S t r i n g ( ) ,   C u l t u r e I n f o . I n v a r i a n t C u l t u r e ) ) ; 
 
                 / / r e t u r n   D o u b l e . P a r s e ( t o k e n . T o S t r i n g ( ) ,   C u l t u r e I n f o . I n v a r i a n t C u l t u r e ) ;   
 
                 r e t u r n   t h i s . t o k e n A s R e a l ; 
 
             } 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   I n d i c a t e s   w h e t h e r   t h e   s p e c i f i e d   c h a r a c t e r   i s   a   c o n t e n t   s t r e a m   w h i t e - s p a c e   c h a r a c t e r . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   s t a t i c   b o o l   I s W h i t e S p a c e ( c h a r   c h ) 
 
         { 
 
             s w i t c h   ( c h ) 
 
             { 
 
                 c a s e   C h a r s . N U L :     / /   0   N u l l 
 
                 c a s e   C h a r s . H T :       / /   9   T a b 
 
                 c a s e   C h a r s . L F :       / /   1 0   L i n e   f e e d 
 
                 c a s e   C h a r s . F F :       / /   1 2   F o r m   f e e d 
 
                 c a s e   C h a r s . C R :       / /   1 3   C a r r i a g e   r e t u r n 
 
                 c a s e   C h a r s . S P :       / /   3 2   S p a c e 
 
                     r e t u r n   t r u e ; 
 
             } 
 
             r e t u r n   f a l s e ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   I n d i c a t e s   w h e t h e r   t h e   s p e c i f i e d   c h a r a c t e r   i s   a n   c o n t e n t   o p e r a t o r   c h a r a c t e r . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   s t a t i c   b o o l   I s O p e r a t o r C h a r ( c h a r   c h ) 
 
         { 
 
             i f   ( C h a r . I s L e t t e r ( c h ) ) 
 
                 r e t u r n   t r u e ; 
 
             s w i t c h   ( c h ) 
 
             { 
 
                 c a s e   C h a r s . A s t e r i s k :         / /   * 
 
                 c a s e   C h a r s . Q u o t e S i n g l e :   / /   ' 
 
                 c a s e   C h a r s . Q u o t e D b l :         / /   " 
 
                     r e t u r n   t r u e ; 
 
             } 
 
             r e t u r n   f a l s e ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   I n d i c a t e s   w h e t h e r   t h e   s p e c i f i e d   c h a r a c t e r   i s   a   P D F   d e l i m i t e r   c h a r a c t e r . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   s t a t i c   b o o l   I s D e l i m i t e r ( c h a r   c h ) 
 
         { 
 
             s w i t c h   ( c h ) 
 
             { 
 
                 c a s e   ' ( ' : 
 
                 c a s e   ' ) ' : 
 
                 c a s e   ' < ' : 
 
                 c a s e   ' > ' : 
 
                 c a s e   ' [ ' : 
 
                 c a s e   ' ] ' : 
 
                 / / c a s e   ' { ' : 
 
                 / / c a s e   ' } ' : 
 
                 c a s e   ' / ' : 
 
                 c a s e   ' % ' : 
 
                     r e t u r n   t r u e ; 
 
             } 
 
             r e t u r n   f a l s e ; 
 
         } 
 
 
 
         p u b l i c   i n t   C o n t L e n g t h 
 
         { 
 
             g e t   {   r e t u r n   t h i s . c o n t e n t . L e n g t h ;   } 
 
         } 
 
 
 
         r e a d o n l y   b y t e [ ]   c o n t e n t ; 
 
         i n t   c h a r I n d e x ; 
 
         c h a r   c u r r C h a r ; 
 
         c h a r   n e x t C h a r ; 
 
 
 
         r e a d o n l y   S t r i n g B u i l d e r   t o k e n   =   n e w   S t r i n g B u i l d e r ( ) ; 
 
         l o n g   t o k e n A s L o n g ; 
 
         d o u b l e   t o k e n A s R e a l ; 
 
         C S y m b o l   s y m b o l   =   C S y m b o l . N o n e ; 
 
     } 
 
 } 
 
 